@@ -6,8 +6,8 @@ This module provides several classes to define a Dashboard.
 
 from powa.json import JSONizable
 from powa.framework import AuthHandler
-from powa.ui_modules import MenuEntry, GlobalMenu
 from powa.compat import with_metaclass, classproperty
+from powa.ui_modules import MenuEntry
 from tornado.web import URLSpec
 from operator import attrgetter
 try:
@@ -34,17 +34,32 @@ class DashboardHandler(AuthHandler):
             self.path_args = args
         params = OrderedDict(zip(self.params,
                                  args))
-        param_dashboard = self.dashboard.parameterized_json(self, **params)
+        param_dashboard = self.dashboard().parameterized_json(self, **params)
         param_datasource = []
         for datasource in self.datasources:
+            # ugly hack to avoid calling the datasource twice per
+            # DashboardPage (once because it's declared in the datasources,
+            # used to automatically register the URLSpecs, and once by the
+            # javascript facility that will look for configuration changes)
+            if datasource.url_name.startswith("datasource_ConfigChanges"):
+                continue
+
             value = datasource.parameterized_json(self, **params)
             value['data_url'] = self.reverse_url(datasource.url_name,
                                                  *args)
             param_datasource.append(value)
+
+        # tell the frontend how to get the configuration changes, if the
+        # DashboardPage provided it
+        param_timeline = None
+        if (self.timeline):
+            param_timeline = self.reverse_url(self.timeline.url_name, *args)
+
         return self.render(self.template,
                            dashboard=param_dashboard,
                            datasources=param_datasource,
-                           title=self.dashboard.title % params)
+                           timeline=param_timeline,
+                           title=self.dashboard().title % params)
 
     @property
     def database(self):
@@ -52,17 +67,10 @@ class DashboardHandler(AuthHandler):
         return params.get("database", None)
 
     @property
-    def menu(self):
+    def breadcrumb(self):
         params = OrderedDict(zip(self.params, self.path_args))
-        new_top_level = list(GlobalMenu)
-        menu = self.get_menu(self, params)
-        for i, child in enumerate(GlobalMenu):
-            if child.url_name == menu.url_name:
-                new_top_level[i] = menu
-                break
-        else:
-            new_top_level.append(menu)
-        return MenuEntry(None, None, None, new_top_level, active=True)
+        breadcrumb = self.get_breadcrumb(self, params)
+        return breadcrumb
 
 
 class MetricGroupHandler(AuthHandler):
@@ -81,12 +89,21 @@ class MetricGroupHandler(AuthHandler):
             for key, value
             in self.request.arguments.items()))
         url_params.update(url_query_params)
+        url_params = self.add_params(url_params)
         query = self.query
-        values = self.execute(query, params=url_params)
-        data = {"data": [self.process(val, **url_params) for val in values]}
+        if (query is not None):
+            values = self.execute(query, params=url_params)
+            data = {"data": [self.process(val, **url_params)
+                             for val in values]}
+        else:
+            data = {}
+
         data = self.post_process(data, **url_params)
         self.render_json(data)
 
+    def add_params(self, params):
+        return params
+
     def process(self, val, **kwargs):
         """
         Callback used to process each individual row fetched from the query.
@@ -121,7 +138,6 @@ class MetricGroupHandler(AuthHandler):
         return data
 
 
-
 class DataSource(JSONizable):
     """
     Base class for various datasources
@@ -166,6 +182,7 @@ class Metric(JSONizable):
         self.label = label or name.title()
         self.yaxis = yaxis or name
         self._group = None
+        self.desc = "No description provided."
         for key, value in kwargs.items():
             setattr(self, key, value)
 
@@ -386,14 +403,19 @@ class Graph(Widget):
     A widget backed by a Rickshaw graph.
     """
 
-    def __init__(self, title, grouper=None,
+    def __init__(self, title, grouper=None, url=None,
                  axistype="time",
                  metrics=None,
                  **kwargs):
         self.title = title
+        self.url = url
         self.grouper = grouper
         self.metrics = metrics or []
         self.axistype = "time"
+        # rickshaw default behavior is to stack metrics, but most of our graphs
+        # use line renderer with unstacked metrics, so by default unstack
+        # metrics, callers will override it if they want
+        self.stack = False
         for key, value in kwargs.items():
             setattr(self, key, value)
 
@@ -544,9 +566,10 @@ class DashboardPage(object):
     base_url = None
     datasources = []
     parent = None
+    timeline = None
 
     @classmethod
-    def url_specs(cls):
+    def url_specs(cls, url_prefix):
         """
         Return the URLSpecs to be register on the application.
         This usually includes one URLSpec for the page itself, and one for
@@ -555,7 +578,7 @@ class DashboardPage(object):
 
         url_specs = []
         url_specs.append(URLSpec(
-            r"%s/" % cls.base_url.rstrip("/"),
+            r"%s%s/" % (url_prefix, cls.base_url.strip("/")),
             type(cls.__name__, (cls.dashboard_handler_cls, cls), {}), {
                 "template": cls.template,
                 "params": cls.params},
@@ -565,39 +588,40 @@ class DashboardPage(object):
                 raise KeyError("A Datasource must have a data_url: %s" %
                                datasource.__name__)
             url_specs.append(URLSpec(
-                datasource.data_url,
+                r"%s%s/" % (url_prefix, datasource.data_url.strip("/")),
                 type(datasource.__name__, (datasource, datasource.datasource_handler_cls),
                      dict(datasource.__dict__)),
                 {"datasource": datasource, "params": cls.params}, name=datasource.url_name))
         return url_specs
 
-    @classmethod
-    def get_menutitle(cls, handler, params):
-        return cls.__name__
-
     @classmethod
     def get_childmenu(cls, handler, params):
-        return []
+        return None
 
     @classmethod
     def get_selfmenu(cls, handler, params):
         my_params = OrderedDict((key, params.get(key))
                                 for key in cls.params)
-        return MenuEntry(cls.get_menutitle(handler, params),
-                         cls.__name__,
-                         my_params)
+        return MenuEntry(cls.title % params, cls.__name__, my_params)
 
     @classmethod
-    def get_menu(cls, handler, params, parent=None):
-        if cls.parent is not None:
-            base = cls.parent.get_menu(handler, params)
-            self_entry = base.findMenu(cls.__name__, params)
-            if self_entry is None:
-                self_entry = cls.get_selfmenu(handler, params)
-                parent = base.findMenu(cls.parent.__name__, params)
-                parent.children.append(self_entry)
+    def get_breadcrumb(cls, handler, params):
+        if (getattr(cls, "breadcrum_title", None)):
+            title = cls.breadcrum_title(handler, params)
         else:
-            self_entry = base = cls.get_selfmenu(handler, params)
-        self_entry.active = True
-        self_entry.children.extend(cls.get_childmenu(handler, params))
-        return base
+            title = cls.title % params
+        entry = MenuEntry(title, cls.__name__, params)
+        entry.children = cls.get_childmenu(handler, params)
+        items = [entry]
+
+        if len(params) > 0:
+            parent_params = params.copy()
+            parent_params.popitem()
+        else:
+            parent_params = []
+
+        if cls.parent is not None and hasattr(handler, 'parent') and \
+            hasattr(cls.parent, 'get_breadcrumb'):
+            items.extend(cls.parent.get_breadcrumb(handler,
+                                                   parent_params))
+        return items
