@@ -16,6 +16,7 @@ extract( epoch from
     ELSE min(total_mesure_interval) END)
 """
 
+
 def unprepare(sql):
     if sql.startswith('PREPARE'):
         sql = re.sub('PREPARE.*AS', '', sql)
@@ -270,7 +271,7 @@ Plan = namedtuple(
     ("title", "values", "query", "plan", "filter_ratio", "exec_count", "occurences"))
 
 
-def qual_constants(type, filter_clause, top=1):
+def qual_constants(srvid, type, tsfrom, tsto, filter_clause, top=1):
     orders = {
         'most_executed': "8 DESC",
         'least_filtering': "9",
@@ -286,47 +287,55 @@ def qual_constants(type, filter_clause, top=1):
     base = text("""
     (
     WITH sample AS (
-    SELECT query, s.queryid, qn.qualid, quals as quals,
+    SELECT s.srvid, query, s.queryid, qn.qualid, quals as quals,
                 constants,
                 sum(occurences) as occurences,
                 sum(execution_count) as execution_count,
                 sum(nbfiltered) as nbfiltered,
                 CASE WHEN sum(execution_count) = 0 THEN 0 ELSE sum(nbfiltered) / sum(execution_count) END AS filter_ratio
         FROM powa_statements s
-        JOIN powa_databases ON powa_databases.oid = s.dbid
-        JOIN powa_qualstats_quals qn ON s.queryid = qn.queryid
+        JOIN powa_databases d ON d.oid = s.dbid AND d.srvid = s.srvid
+        JOIN powa_qualstats_quals qn ON s.queryid = qn.queryid AND s.srvid = qn.srvid
         JOIN (
             SELECT *
             FROM powa_qualstats_constvalues_history qnc
+            WHERE srvid = :server
+            AND coalesce_range && tstzrange(:from, :to)
             UNION ALL
             SELECT *
-            FROM powa_qualstats_aggregate_constvalues_current
-        ) qnc ON qn.qualid = qnc.qualid AND qn.queryid = qnc.queryid,
+            FROM powa_qualstats_aggregate_constvalues_current(:server, :from, :to)
+            WHERE srvid = :server
+        ) qnc ON qnc.srvid = s.srvid AND qn.qualid = qnc.qualid AND qn.queryid = qnc.queryid,
         LATERAL
-                unnest(%s) as t(constants,occurences, nbfiltered,execution_count)
-        WHERE %s
-        GROUP BY qn.qualid, quals, constants, s.queryid, query
-        ORDER BY %s
+                unnest(%(qual_type)s) as t(constants,occurences, execution_count, nbfiltered)
+        WHERE %(filter)s
+        AND s.srvid = :server
+        GROUP BY s.srvid, qn.qualid, quals, constants, s.queryid, query
+        ORDER BY %(order)s
         LIMIT :top_value
     )
-    SELECT query, queryid, qualid, quals, constants as constants,
+    SELECT srvid, query, queryid, qualid, quals, constants as constants,
                 occurences as occurences,
                 nbfiltered as nbfiltered,
                 execution_count as execution_count,
                 filter_ratio as filter_ratio,
                 row_number() OVER (ORDER BY execution_count desc NULLS LAST) as rownumber
         FROM sample
-    ORDER BY 10
+    ORDER BY 11
     LIMIT :top_value
-    ) %s
-    """ % (type, str(filter_clause), orders[type], type)
-               )
+    ) %(qual_type)s
+    """ % {
+        'qual_type': type,
+        'filter': str(filter_clause),
+        'order': orders[type]})
     base = base.params(top_value=top, **filter_clause.params)
     return select(["*"]).select_from(base)
 
+
 def quote_ident(name):
     return '"' + name + '"'
 
+
 def get_plans(self, query, database, qual):
     plans = []
     for key in ('most filtering', 'least filtering', 'most executed'):
@@ -335,9 +344,10 @@ def get_plans(self, query, database, qual):
         plan = "N/A"
         try:
             result = self.execute("EXPLAIN %s" % query,
-                                  database=database)
+                                  database=database,
+                                  remote_access=True)
             plan = "\n".join(v[0] for v in result)
-        except:
+        except Exception:
             pass
         plans.append(Plan(key, vals['constants'], query,
                           plan, vals["filter_ratio"], vals['execution_count'],
@@ -345,7 +355,7 @@ def get_plans(self, query, database, qual):
     return plans
 
 
-def get_unjumbled_query(ctrl, database, queryid, _from, _to,
+def get_unjumbled_query(ctrl, srvid, database, queryid, _from, _to,
                         kind='most executed'):
     """
     From a queryid, build a query string ready to be executed.
@@ -353,17 +363,21 @@ def get_unjumbled_query(ctrl, database, queryid, _from, _to,
     Gather a jumbled query from powa_statements, then try to denormalized it
     using stored const values, by its kind (least/most filtering, most common).
 
-    This function can return None if the query has not been gathered by powa, or
-    a partially or fully normalized query, depending on const values has been
-    found and/or the SELECT clause has been normalized
+    This function can return None if the query has not been gathered by powa,
+    or a partially or fully normalized query, depending on const values has
+    been found and/or the SELECT clause has been normalized
     """
 
     rs = list(ctrl.execute(text("""
-        SELECT query FROM powa_statements WHERE queryid = :queryid LIMIT 1
-    """), params={"queryid": queryid}))[0]
+        SELECT query
+        FROM powa_statements
+        WHERE srvid= :srvid
+        AND queryid = :queryid LIMIT 1
+    """), params={"srvid": srvid, "queryid": queryid}))[0]
     normalized_query = rs[0]
-    values = qualstat_get_figures(ctrl, database, _from, _to,
+    values = qualstat_get_figures(ctrl, srvid, database, _from, _to,
                                   queries=[queryid])
+
     if values is None:
         unprepared = unprepare(normalized_query)
         if unprepared != normalized_query:
@@ -374,7 +388,8 @@ def get_unjumbled_query(ctrl, database, queryid, _from, _to,
                                values[kind].get('constants', []))
     return sql
 
-def get_any_sample_query(ctrl, database, queryid, _from, _to):
+
+def get_any_sample_query(ctrl, srvid, database, queryid, _from, _to):
     """
     From a queryid, get a non normalized query.
 
@@ -384,7 +399,7 @@ def get_any_sample_query(ctrl, database, queryid, _from, _to):
     If this fail, fallback get_unjumbled_query, with "most executed" const
     values.
     """
-    has_pgqs = ctrl.has_extension("pg_qualstats")
+    has_pgqs = ctrl.has_extension_version(srvid, "pg_qualstats")
     example_query = None
     if has_pgqs and has_pgqs >= "0.0.7":
         rs = list(ctrl.execute(text("""
@@ -392,20 +407,25 @@ def get_any_sample_query(ctrl, database, queryid, _from, _to):
             FROM powa_statements
             WHERE queryid = :queryid
             LIMIT 1
-        """), params={"queryid": queryid}))[0]
+        """), params={"queryid": queryid}, remote_access=True))[0]
         example_query = rs[1]
         if example_query is not None:
             unprepared = unprepare(example_query)
             if example_query == unprepared:
                 return example_query
-    return get_unjumbled_query(ctrl, database, queryid,
+    return get_unjumbled_query(ctrl, srvid, database, queryid,
                                _from, _to, 'most executed')
 
-def qualstat_get_figures(conn, database, tsfrom, tsto, queries=None, quals=None):
-    condition = text("""datname = :database AND coalesce_range && tstzrange(:from, :to)""")
+
+def qualstat_get_figures(conn, srvid, database, tsfrom, tsto,
+                         queries=None, quals=None):
+    condition = text("""datname = :database
+            AND coalesce_range && tstzrange(:from, :to)""")
+
     if queries is not None:
         condition = and_(condition, array([int(q) for q in queries])
                          .any(literal_column("s.queryid")))
+
     if quals is not None:
         condition = and_(condition, array([int(q) for q in quals])
                          .any(literal_column("qnc.qualid")))
@@ -418,26 +438,30 @@ def qualstat_get_figures(conn, database, tsfrom, tsto, queries=None, quals=None)
         text('to_json(most_executed) as "most executed"'),
         text('to_json(most_used) as "most used"')])
            .select_from(
-               qual_constants("most_filtering", condition)
+               qual_constants(srvid, "most_filtering", tsfrom, tsto, condition)
                .alias("most_filtering")
                .join(
-                   qual_constants("least_filtering", condition)
+                   qual_constants(srvid, "least_filtering",
+                                  tsfrom, tsto, condition)
                    .alias("least_filtering"),
                    text("most_filtering.rownumber = "
                         "least_filtering.rownumber"))
-               .join(qual_constants("most_executed", condition)
+               .join(qual_constants(srvid, "most_executed",
+                                    tsfrom, tsto, condition)
                      .alias("most_executed"),
                      text("most_executed.rownumber = "
                           "least_filtering.rownumber"))
-               .join(qual_constants("most_used", condition)
+               .join(qual_constants(srvid, "most_used",
+                                    tsfrom, tsto, condition)
                      .alias("most_used"),
                      text("most_used.rownumber = "
                           "least_filtering.rownumber"))))
 
-
-    params = {"database": database,
+    params = {"server": srvid,
+              "database": database,
               "from": tsfrom,
-              "to": tsto}
+              "to": tsto,
+              "queryids": queries}
     quals = conn.execute(sql, params=params)
 
     if quals.rowcount == 0:
@@ -469,6 +493,7 @@ class HypoPlan(JSONizable):
         base['gain_percent'] = self.gain_percent
         return base
 
+
 class HypoIndex(JSONizable):
 
     def __init__(self, nspname, relname, amname,
@@ -495,7 +520,6 @@ class HypoIndex(JSONizable):
                     quote_ident(self.relname),
                     ",".join(attrs)))
 
-
     def __setattr(self, name, value):
         super(HypoIndex, self).__setattr__(name, value)
         # Only btree is supported right now
@@ -510,7 +534,8 @@ class HypoIndex(JSONizable):
     def hypo_ddl(self):
         ddl = self.ddl
         if ddl is not None:
-            return select([column("indexname")]).select_from(func.hypopg_create_index(self.ddl))
+            return select([column("indexname")]
+                          ).select_from(func.hypopg_create_index(self.ddl))
 
     def to_json(self):
         base = super(HypoIndex, self).to_json()
@@ -520,6 +545,7 @@ class HypoIndex(JSONizable):
 
 def possible_indexes(composed_qual, order=()):
     by_am = defaultdict(list)
+
     def sorter(qual):
         attnum = qual.attnum
         if attnum in order:
@@ -567,6 +593,6 @@ def get_hypoplans(conn, query, indexes=None):
     hypocost = float(m.group(0))
     used_indexes = []
     for ind in indexes:
-        if ind.name != None and ind.name in hypoplan:
+        if ind.name is not None and ind.name in hypoplan:
             used_indexes.append(ind)
     return HypoPlan(baseplan, basecost, hypoplan, hypocost, query, used_indexes)
